@@ -154,12 +154,27 @@ def load_patch_image(dataset,imList):
 
     labels = []
     last_patch = 0
+    x_y_shape = [0,0]
     for im_i in imList:
+
+        # Create label image
         num_patches = int(im_i.shape[0]/dataset.patch_size)*int(im_i.shape[1]/dataset.patch_size)
         labels.append(np.transpose(np.reshape(np.array(range(last_patch,last_patch+num_patches)),(int(im_i.shape[1]/dataset.patch_size),int(im_i.shape[0]/dataset.patch_size)))))
         last_patch = last_patch+num_patches
 
-    return np.concatenate(labels,1), labels
+        # Update shape of mosaic
+        x_y_shape[0], x_y_shape[1] = max(labels[-1].shape[0],x_y_shape[0]), labels[-1].shape[1]+x_y_shape[1]
+            
+    # Create mosaic
+    mosaic = np.zeros(x_y_shape,dtype=np.uint32)+4294967295
+
+    # Concatenate images
+    col=0
+    for lab in labels:
+        mosaic[0:lab.shape[0],col:col+lab.shape[1]] = lab
+        col += lab.shape[1]
+
+    return mosaic, labels
 
 def select_patches_from_cohort_(clusters,dataset,subject_info,count):
 
@@ -257,7 +272,7 @@ def topk_confident_patches(dataset,n_cell_types,
                             CropConfTissueComm,im,imList,PCL_reprsntions,count):
     '''
     '''
-    K = int(np.ceil(5/dataset.processed_file_names*100))
+    K = int(np.ceil(10/dataset.processed_file_names*100))
 
     # Create label map. 
     labels,labels_list = load_patch_image(dataset,imList)
@@ -329,10 +344,10 @@ def calculate_marker_colocalization(matrix,MarkerNames):
         id_1 = MarkerNames.index(pair_of_markers[1])
         matrix_0 = matrix[:,:,:,[id_0]]
         matrix_1 = matrix[:,:,:,[id_1]]
-        for n_patch in range(matrix.shape[0]):            
-            Marker_Colocalization[id_0,id_1] += calculate_pearson_correlation(matrix_0[n_patch,:,:],matrix_1[n_patch,:,:])
+        # for n_patch in range(matrix.shape[0]):            
+        Marker_Colocalization[id_0,id_1] = calculate_pearson_correlation(matrix_0,matrix_1)
         Marker_Colocalization[id_1,id_0] = Marker_Colocalization[id_0,id_1]
-    return Marker_Colocalization/matrix.shape[0]
+    return Marker_Colocalization
 
 def calculate_Cell_Size_circularity(filename, matrix):
     # matrix: contains (number of patches, x_dimension, y_dimension)         
@@ -360,10 +375,14 @@ def calculate_Cell_Size_circularity(filename, matrix):
                     Granularity[mk] += output[0]-1
 
                     count += 1
-        
-        Size[mk] /= count
-        Eccentricity[mk] /= count
-        Granularity[mk] /= count
+        if count>0:
+            Size[mk] /= count
+            Eccentricity[mk] /= count
+            Granularity[mk] /= count
+        else:
+            Size[mk] = 0
+            Eccentricity[mk] = 0
+            Granularity[mk] = 0
 
     return Size, Eccentricity, Granularity
 
@@ -450,9 +469,9 @@ def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker
     # Figure Heatmap Raw Values
     plt.close('all')
     plt.figure()
-    sns.set(font_scale=1.1)
+    sns.set(font_scale=0.9)
     cluster=True if (heatmap.sum(1)!=0).sum()>1 else False
-    h_E_Fig = sns.clustermap(heatmap[heatmap.sum(1)!=0,:],col_cluster=cluster, row_cluster=cluster, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+    h_E_Fig = sns.clustermap(heatmap[heatmap.sum(1)!=0,:],col_cluster=cluster, row_cluster=cluster, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels,vmin=-1,vmax=1, linewidths=0.5, cmap="bwr")            
     h_E_Fig.savefig(filename+'_Raw.png',dpi=600) 
 
     # Save Excel.
@@ -466,14 +485,14 @@ def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker
     h_E_COL_MinMax[np.isnan(h_E_COL_MinMax)] = 0 
     plt.close('all')
     plt.figure()
-    sns.set(font_scale=1.1)
+    sns.set(font_scale=0.9)
     if (h_E_COL_MinMax.sum(1)!=0).sum()==1: # One column heatmap
         h_E_Fig = sns.clustermap(h_E_COL_MinMax[h_E_COL_MinMax.sum(1)!=0,:],col_cluster=False, 
-            row_cluster=False, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+            row_cluster=False, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5,vmin=-1,vmax=1, cmap="bwr")            
         h_E_Fig.savefig(filename+'_MinMax.png',dpi=600) 
     elif (h_E_COL_MinMax.sum(1)!=0).sum()>1: # Normal heatmap.    
         h_E_Fig = sns.clustermap(h_E_COL_MinMax[h_E_COL_MinMax.sum(1)!=0,:],col_cluster=True, 
-            row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+            row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5,vmin=-1,vmax=1, cmap="bwr")            
         h_E_Fig.savefig(filename+'_MinMax.png',dpi=600)  
     
     # Figure Heatmap z-scored values
@@ -481,12 +500,12 @@ def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker
     h_E_COL_Norm[np.isnan(h_E_COL_Norm)] = 0              
     plt.close('all')
     plt.figure()
-    sns.set(font_scale=1.1)    
+    sns.set(font_scale=0.9)    
     if (h_E_COL_Norm.sum(1)!=0).sum()==1: # One column heatmap
-        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=False, vmin=-2, vmax=2, row_cluster=False, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=False, vmin=-1, vmax=1, row_cluster=False, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
         h_E_Fig.savefig(filename+'_Norm.png',dpi=600) 
     elif (h_E_COL_Norm.sum(1)!=0).sum()>1: # Normal heatmap
-        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=True, vmin=-2, vmax=2, row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=True, vmin=-1, vmax=1, row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
         h_E_Fig.savefig(filename+'_Norm.png',dpi=600) 
 
     # Figure Heatmap z-scored values
@@ -494,12 +513,12 @@ def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker
     h_E_COL_Norm[np.isnan(h_E_COL_Norm)] = 0              
     plt.close('all')
     plt.figure()
-    sns.set(font_scale=1.1)    
+    sns.set(font_scale=0.9)    
     if (h_E_COL_Norm.sum(1)!=0).sum()==1: # One column heatmap
-        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=False, vmin=-2, vmax=2, row_cluster=False, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=False, vmin=-1, vmax=1, row_cluster=False, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
         h_E_Fig.savefig(filename+'_Norm_rows.png',dpi=600) 
     elif (h_E_COL_Norm.sum(1)!=0).sum()>1: # Normal heatmap
-        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=True, vmin=-2, vmax=2, row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
+        h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=True, vmin=-1, vmax=1, row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
         h_E_Fig.savefig(filename+'_Norm_rows.png',dpi=600) 
 
 def save_heatMapMarker_and_barplot(dataset, heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity, CropConf,Marker_Names,cell_type,Thresholds):
@@ -507,37 +526,61 @@ def save_heatMapMarker_and_barplot(dataset, heatmapMarkerExpression, heatmap_Col
     '''
     if cell_type=='Phenotypes':
         abrev = 'P'
+        names = np.array([abrev+str(i+1) for i in range(len(CropConf))])
+
+        # Exclude phenotypes from the analysis
+        phnos = np.array(range(heatmapMarkerExpression.shape[0]))
+        for excluded in dataset.args['Pheno_Excluded_from_analysis']:            
+            phnos = phnos[phnos!=excluded-1]
+        heatmapMarkerExpression = heatmapMarkerExpression[phnos,:]
+        heatmap_Size = heatmap_Size[phnos,:]
+        heatmap_Eccentricity = heatmap_Eccentricity[phnos,:]
+        heatmap_Granularity = heatmap_Granularity[phnos,:]
+        names = names[phnos]
+        
     else:
         abrev = 'N'
+        names = np.array([abrev+str(i+1) for i in range(len(CropConf))])
+
+        # Exclude phenotypes from the analysis
+        neighs = np.array(range(heatmapMarkerExpression.shape[0]))
+        for excluded in dataset.args['Neigh_Excluded_from_analysis']:            
+            neighs = neighs[neighs!=excluded-1]
+        heatmapMarkerExpression = heatmapMarkerExpression[neighs,:]
+        heatmap_Size = heatmap_Size[neighs,:]
+        heatmap_Eccentricity = heatmap_Eccentricity[neighs,:]
+        heatmap_Granularity = heatmap_Granularity[neighs,:]
+        names = names[neighs]
 
     Colormap=cm.jet(range(0,255,int(255/heatmapMarkerExpression.shape[0])))[:,:3]
 
     # Save heatmapmarker to disk    
     makedirs(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Expression/')
     save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Expression/heatmap', 
-                                        heatmapMarkerExpression, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        heatmapMarkerExpression, names,
                                         Colormap,Marker_Names)
 
     makedirs(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/')
     save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/Heatmap_Size', 
-                                        heatmap_Size, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        heatmap_Size, names,
                                         Colormap,Marker_Names)            
 
     save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/heatmap_Eccentricity', 
-                                        heatmap_Eccentricity, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        heatmap_Eccentricity, names,
                                         Colormap,Marker_Names)         
 
     save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/heatmap_Granularity', 
-                                        heatmap_Eccentricity, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        heatmap_Granularity, names,
                                         Colormap,Marker_Names)     
 
     # Correlation matrix
     makedirs(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Colocalization/')
     mask = np.triu(np.ones_like(heatmap_Colocalization[0,:,:], dtype=np.bool))
+    sns.set(font_scale=0.5)    
     for TME in range(len(heatmap_Colocalization)):
         plt.close('all')
         plt.figure()
-        heatmap = sns.heatmap(heatmap_Colocalization[TME,:,:], mask=mask, vmin=heatmap_Colocalization.min(), vmax=heatmap_Colocalization.max(),xticklabels=Marker_Names,yticklabels=Marker_Names, annot=True, cmap='BrBG')
+        heatmap = sns.heatmap(heatmap_Colocalization[TME,:,:], mask=mask, vmin=heatmap_Colocalization.min(), vmax=heatmap_Colocalization.max(),xticklabels=Marker_Names,yticklabels=Marker_Names, cmap='BrBG')
         heamap = heatmap.get_figure()
         heamap.savefig(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Colocalization/heatmap_Colocalization_'+abrev+str(TME+1)+'.png',dpi=600) 
         df = pd.DataFrame(heatmap_Colocalization[TME,:,:], columns = Marker_Names)
@@ -637,7 +680,7 @@ def Area_to_Neighborhood_to_Phenotype(dataset,clusters,IndexAndClass, Clustering
     h_E_Fig.savefig(dataset.bioInsights_dir_cell_types_Areas + 'heatmap_Neighborhood_composition_of_Areas_Perc_'+str(ClusteringThreshold)+'.png',dpi=600) 
     plt.close('all')
     plt.figure()
-    h_E_Fig = sns.clustermap(Area_to_Neigh.transpose(),col_cluster=True, row_cluster=True, col_colors=row_colors_Area, z_score=1,
+    h_E_Fig = sns.clustermap(Area_to_Neigh.transpose(),col_cluster=True, row_cluster=True,vmin=-1, vmax=1, col_colors=row_colors_Area, z_score=1,
                                 row_colors=col_colors_Neigh,yticklabels=xticklabels_Neigh,xticklabels=yticklabels_Area, linewidths=0.5, cmap="bwr")            
     h_E_Fig.savefig(dataset.bioInsights_dir_cell_types_Areas + 'heatmap_Neighborhood_composition_of_Areas_Zscore_'+str(ClusteringThreshold)+'.png',dpi=600) 
     df = pd.DataFrame(Area_to_Neigh.transpose(), columns = yticklabels_Area)
@@ -671,7 +714,7 @@ def Area_to_Neighborhood_to_Phenotype(dataset,clusters,IndexAndClass, Clustering
     plt.close('all')
     plt.figure()
     Neigh_to_Pheno[Neigh_to_Pheno==0] = 1e-16
-    h_E_Fig = sns.clustermap(Neigh_to_Pheno.transpose(),col_cluster=True, row_cluster=True, col_colors=row_colors_Neigh, z_score=1,
+    h_E_Fig = sns.clustermap(Neigh_to_Pheno.transpose(),col_cluster=True, row_cluster=True,vmin=-1, vmax=1, col_colors=row_colors_Neigh, z_score=1,
                                 row_colors=col_colors_Pheno,yticklabels=xticklabels_Pheno,xticklabels=yticklabels_Neigh, linewidths=0.5, cmap="bwr")            
     h_E_Fig.savefig(dataset.bioInsights_dir_cell_types + 'Neighborhoods/heatmap_Phenotype_composition_of_neighborhoods_Zscore_Thrs{}.png'.format(str(ClusteringThreshold)),dpi=600) 
     df = pd.DataFrame(Neigh_to_Pheno.transpose(), columns = yticklabels_Neigh)
