@@ -642,26 +642,6 @@ class NaroNet_dataset(torch.utils.data.Dataset):
                         # Nothing = statisticalTest.append([stats.kruskal(groups['1'], groups['2'])[1],['1','2'], heatmap,idxColumn]) if stats.kruskal(groups['1'], groups['2'])[1]<pvalueThreshold else 0
                         Nothing = statisticalTest.append([stats.kruskal(groups['0'],groups['1'], groups['2'])[1],['0','1','2'], heatmap,idxColumn]) if stats.kruskal(groups['0'],groups['1'], groups['2'])[1]<pvalueThreshold else 0
         return statisticalTest
-    
-    # def FindMultivariateDifferencesBetweenGroups(self, heatmapALL, labels, IndexAndClass):        
-    #     pvalueThreshold=0.001
-    #     statisticalTest = [] # List of columns with statistical difference. 1.p-value, 2.Cluster step, 3.column of the heatmap.
-        
-    #     # Aggregate each heatmap from the cluster's hierarchy
-    #     for heatmap in heatmapALL:
-            
-    #         # Create Model to find the parameters that most define the disease.
-    #         model = smapi.OLS(np.array(labels[[heatmap for heatmap in heatmapALL][-1]]), heatmapALL[heatmap])
-    #         results = model.fit()
-            
-    #         # F-Test of the Model.
-    #         FTest=results.f_test(np.identity(results.params.shape[0]))
-            
-    #         # Obtain most significant clusters
-    #         SignificantClusters = list(np.where(results.pvalues<pvalueThreshold)[0])
-    #         for sC in SignificantClusters:
-    #             statisticalTest.append([results.pvalues[sC],heatmap,sC])
-    #     return statisticalTest
 
     def open_Raw_Image(self, idxclster,reverseIndx):
         Channels, Marker_Names = utilz.load_channels(self.root+'Raw_Data/')
@@ -674,7 +654,10 @@ class NaroNet_dataset(torch.utils.data.Dataset):
             patient_to_image_excel = pd.read_excel(self.root+'Raw_Data/Experiment_Information/Patient_to_Image.xlsx')  
 
             # Find the actual patient
-            image_indices = patient_to_image_excel['Image_Name'][patient_to_image_excel['Subject_Name'].isin([idxclster[0]])]            
+            image_indices = patient_to_image_excel['Image_Name'][patient_to_image_excel['Subject_Name'].isin([idxclster[0]])]       
+
+            # Shape of images
+            x_y_c_shape = [0, 0, 0]
             
             # Iterate through images corresponding to this patient
             for numImages, im_name in enumerate(list(image_indices)):
@@ -701,9 +684,20 @@ class NaroNet_dataset(torch.utils.data.Dataset):
                 
                 # Append to list of images
                 imList.append(im)
-            im = np.concatenate(imList,axis=1)            
+
+                # Upadte sizes of mosaic
+                x_y_c_shape[0], x_y_c_shape[1], x_y_c_shape[2] = max(im.shape[0],x_y_c_shape[0]), im.shape[1]+x_y_c_shape[1], max(im.shape[2],x_y_c_shape[2])
+            
+            # Create mosaic
+            im = np.zeros(x_y_c_shape)
+
+            # Concatenate images
+            col=0
+            for im_i in imList:
+                im[0:im_i.shape[0],col:col+im_i.shape[1],0:im_i.shape[2]] = im_i
+                col += im_i.shape[1]
                 
-        else:
+        else:   
             # Load Image in its own format.
             if os.listdir(self.root+'Raw_Data/Images/')[-1].split('.')[-1]=='tiff':
                 im = tifffile.imread(self.root+'Raw_Data/Images/'+idxclster[0])                 
@@ -730,47 +724,6 @@ class NaroNet_dataset(torch.utils.data.Dataset):
             imList.append(im)
         return im, imList
 
-    def nPlex2RGB(self, im):
-        if ('Endometrial' in self.root) or ('Cytof52Breast' in self.root) or ('ZuriBasel' in self.root):
-            imRGB = np.zeros((im.shape[0],im.shape[1],3))                        
-            # im = im/im.max((0,1),keepdims=True)
-            imRGB[:,:,2] += im[:,:,0] # DAPI is blue
-            imRGB[:,:,0] += im[:,:,1] # 2ndcolr is red
-            imRGB[:,:,0] += im[:,:,2]*1 # 3rdcolr is yellow
-            imRGB[:,:,1] += im[:,:,2]*1 # 3rdcolr is yellow
-            imRGB[:,:,0] += im[:,:,3]*1 # 4thcolr is Orange
-            imRGB[:,:,1] += im[:,:,3]*0.5 # 4thcolr is Orange
-            imRGB[:,:,0] += im[:,:,4]*1 # 5thcolr is Magenta
-            imRGB[:,:,2] += im[:,:,4]*1 # 5thcolr is Magenta                     
-            imRGB[:,:,1] += im[:,:,5] # 6thcolr is Green      
-            imRGB[:,:,1] += im[:,:,6]*1 # 7thcolr is Cyan
-            imRGB[:,:,2] += im[:,:,6]*1 # 7thcolr is Cyan 
-            for i in range(3):
-                imRGB[:,:,i] = imRGB[:,:,i]/imRGB[:,:,i].max()
-        elif 'Synthetic' in self.root:
-            imRGB = np.zeros((im.shape[0],im.shape[1],3))                        
-            im = im -im.min()
-            imRGB[:,:,2] += im[:,:,0] # DAPI is blue
-            imRGB[:,:,0] += im[:,:,1] # 2ndcolr is red
-            imRGB[:,:,0] += im[:,:,2]*1 # 3rdcolr is yellow
-            imRGB[:,:,1] += im[:,:,2]*1 # 3rdcolr is yellow
-            imRGB[:,:,0] += im[:,:,3]*1 # 4thcolr is Orange
-            imRGB[:,:,1] += im[:,:,3]*0.5 # 4thcolr is Orange
-            imRGB[:,:,0] += im[:,:,4]*1 # 5thcolr is Magenta
-            imRGB[:,:,2] += im[:,:,4]*1 # 5thcolr is Magenta                     
-            imRGB[:,:,1] += im[:,:,5] # 6thcolr is Green      
-            for i in range(3):
-                imRGB[:,:,i] = imRGB[:,:,i]/imRGB[:,:,i].max()
-        elif 'AlfonsoCalvo' in self.root:
-            imRGB = np.zeros((im.shape[0],im.shape[1],3))                        
-            imRGB[:,:,2] += im[:,:,0] # DAPI is blue
-            imRGB[:,:,0] += im[:,:,2] # CD8 is red                                                                
-            imRGB[:,:,0] += im[:,:,3]*0.5 # 5thcolr is Magenta
-            imRGB[:,:,2] += im[:,:,3]*0.5 # 5thcolr is Magenta                     
-            imRGB[:,:,1] += im[:,:,1] # CD4 is Green      
-            for i in range(3):
-                imRGB[:,:,i] = imRGB[:,:,i]/imRGB[:,:,i].max()
-        return imRGB
 
     
 
