@@ -336,7 +336,10 @@ def parameters(path, debug):
         
         # Bioinsights parameters
         args['Bio_save_Orig_im'] = True
-        args['TSNE_Perc_Pat'] = 1 # Percentage of patients to use to generate the tsne. 
+        args['TSNE_Perc_Pat'] = 0 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Pheno_Perc_Pat'] = 1 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Pheno_Excluded_from_analysis'] = [] # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Neigh_Excluded_from_analysis'] = [] # Percentage of patients used to calculate TSNE. 1 is 100% of patients
 
 
     elif 'Endometrial_Survival' in path:            
@@ -676,6 +679,80 @@ def parameters(path, debug):
         args['Bio_save_Orig_im'] = True
         args['TSNE_Perc_Pat'] = 0 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
         args['Pheno_Perc_Pat'] = 1 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Pheno_Excluded_from_analysis'] = [25] # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Neigh_Excluded_from_analysis'] = [] # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+    
+    elif 'Pancreas_Adenocarcinoma' in path:            
+        
+        # Patch contrastive learning parameters
+        args['PCL_CNN_Architecture']='resnet50' # or resnet18
+        args['PCL_N_Workers']=10 # Number of data loading workers 
+        args['PCL_Epochs']=600 # Number of total epoch to run
+        args['PCL_N_Crops_per_Image']=40 # Number of crops chosen per image
+        args['PCL_Patch_Size']=30 # Size of patches in pixels
+        args['PCL_Stride']=0 # Size of stride between consecutive patches
+        args['PCL_Alpha_L']=1.15 # size ratio between image crop and image patch
+        args['PCL_Z_Score']= True # Whether to apply z score per channel/marker
+        args['PCL_Batch_Size']= 20 # Number of images per iteration
+        args['PCL_Weight_Decay']= 1e-4 # Weight Decay
+        args['PCL_Out_Dimensions']= 128 # Feature dimensions of patch
+        args['PCL_Learning_Rate']= 0.0003 # Learning Rate
+        args['PCL_Temperature']= 0.07 # Softmax temperature                
+        args['PCL_eliminate_Black_Background'] = False
+        args['PCL_eliminate_White_Background'] = True
+        args['PCL_GPU_INDEX']= torch.device('cuda:0') # GPU where the training is carried out 
+        args['PCL_Color_perturbation_Augmentation'] = False # Color augmentation for H&E tissue staining.
+
+        # Label you want to infer with respect the images.
+        args['experiment_Label'] = ['Patient_Status'] 
+
+        # Architecture Search parameters
+        args['num_samples_architecture_search'] = 1500  
+
+        # Optimization Parameters
+        args['NaroNet_n_workers'] = 1
+        args['epochs'] = 1000
+        args['lr_decay_factor'] = 0.1
+        args['lr_decay_step_size'] = 10000
+        args['weight_decay'] = 0
+        args['batch_size'] = 24
+        args['lr'] = 0.0001 
+
+        # General
+        args['training_MODE'] = 'TrainALL' # TrainALL, NestedCrossValidation, CrossValidation # Whether to use nested cross validation or cross validation.        
+        args['folds'] = 1
+        args['device'] = 'cuda:0'
+        args['Batch_Normalization'] = True
+        args['dataAugmentationPerc'] = 0 #0 if debug=='Index' else hp.choice("dataAugmentationPerc", [0,0.0001,0.001]) if debug=='Object' else 0  
+
+        # Neural Network
+        args['N_Phenotypes'] = 8
+        args['N_Neighborhoods'] = 12
+        args['N_Areas'] = 4
+        args['n_hops_neighborhoods'] = 2
+        args['Phenotype_Learning'] = True  
+        args['Neighborhood_Learning'] = True
+        args['Area_Learning'] = False
+        args['dropoutRate'] = 0# 1 if debug=='Index' else hp.choice('dropoutRate', [0.1, 0.2, 0.3,0.4]) if debug=='Object' else 0.2        
+        args['attntnThreshold'] = 0# if debug=='Index' else hp.choice('attntnThreshold', [0,.2,.4,.6,.8]) if debug=='Object' else 0  
+        args['1Patch1Cluster'] = False
+                
+        # Losses
+        args['ContrastACC_Pheno'] = 1000#4 if debug=='Index' else hp.choice("orthoColor_Lambda0", [1,0.1,0.01,0.001,0.0001,0.00001]) if debug=='Object' else 0.0001                                
+        args['ContrastACC_Neigh'] = 1000#4 if debug=='Index' else hp.choice("orthoColor_Lambda1", [1, 0.1,0.01,0.001,0.0001,0.00001]) if debug=='Object' else 0.0001                                              
+        args['ContrastACC_Area'] = 0#4 if debug=='Index' else hp.choice("orthoColor_Lambda1", [1, 0.1,0.01,0.001,0.0001,0.00001]) if debug=='Object' else 0.0001     
+        args['Perc_high_conf_patches'] = 0.0001 # 1 is 100%. Great values generates phenotypes and neighborhoods with many patches. Low values generate rare populations.
+        args['PatchEntropy_Pheno'] = 0#3 if debug=='Index' else hp.choice("min_Cell_entropy_Lambda0", [1,0.1,0.01,0.001,0]) if debug=='Object' else 0.001   
+        args['PatchEntropy_Neigh'] = 0#4 if debug=='Index' else hp.choice("min_Cell_entropy_Lambda1", [1,0.1,0.01,0.001,0]) if debug=='Object' else 0
+        args['PatchEntropy_Area'] = 0#1 if debug=='Index' else hp.choice("min_Cell_entropy_Lambda2", [1,0.1,0.01,0.001,0]) if debug=='Object' else 0.1          
+        
+        # Bioinsights parameters
+        args['Bio_save_Orig_im'] = True
+        args['TSNE_Perc_Pat'] = 0 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Pheno_Perc_Pat'] = 1 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Pheno_Excluded_from_analysis'] = [] # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Neigh_Excluded_from_analysis'] = [] # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+
 
     else:
         
